"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const index_1 = require("./index");
const sleepValue = (millis, value) => new Promise(r => setTimeout(() => r(value), millis));
const sleepError = (millis, error) => new Promise((_, r) => setTimeout(() => r(error), millis));
describe('wait', () => {
    it('should return the promise resolved value', async () => {
        expect.assertions(2);
        await index_1.resync(() => {
            let value = index_1.wait(() => sleepValue(10, 7));
            index_1.affect(() => expect(value).toEqual(7));
            value = index_1.wait(() => sleepValue(10, 13));
            index_1.affect(() => expect(value).toEqual(13));
        })();
    });
    it('should throw the promise rejected value', async () => {
        await index_1.resync(() => {
            try {
                index_1.wait(() => sleepError(10, new Error("Foo")));
                fail('it should not reach here');
            }
            catch (e) {
                index_1.except(e);
                index_1.wait(() => sleepValue(10));
            }
        })();
    });
});
describe('resync', () => {
    it('should return', async () => {
        const val = await index_1.resync(() => {
            return index_1.wait(() => sleepValue(10, 7));
        })();
        expect(val).toEqual(7);
    });
    it('should throw', async () => {
        expect.assertions(1);
        await index_1.resync(() => {
            return index_1.wait(() => sleepError(10, new Error('error')));
        })().catch(e => expect(e).toBeInstanceOf(Error));
    });
    it('should run in parallel', async () => {
        const sum = index_1.resync((a, b) => {
            const futureA = index_1.wait(() => sleepValue(10, a));
            const futureB = index_1.wait(() => sleepValue(10, b));
            return index_1.wait(() => sleepValue(10, futureA + futureB));
        });
        const [x, y] = await Promise.all([sum(3, 5), sum(13, 7)]);
        expect(x).toEqual(8);
        expect(y).toEqual(20);
    });
});
describe('state', () => {
    it('should run stateful methods once and only once', async () => {
        let executions = 0;
        await index_1.resync(() => {
            index_1.wait(() => sleepValue(10));
            index_1.affect(() => executions++);
            index_1.wait(() => sleepValue(10));
            index_1.wait(() => sleepValue(10));
        })();
        expect(executions).toEqual(1);
    });
    it('should always return stateful value', async () => {
        await index_1.resync(() => {
            const val = index_1.state(() => 7);
            expect(val).toEqual(7);
            index_1.wait(() => sleepValue(10));
        })();
    });
    it('should always throw stateful value', async () => {
        await index_1.resync(() => {
            try {
                index_1.state(() => { throw new Error("Foo"); });
                fail('it should not reach here');
            }
            catch (e) {
                index_1.except(e);
                index_1.wait(() => sleepValue(10));
            }
        })();
    });
    it('should support being nested', async () => {
        const noop = () => { };
        const val = await index_1.resync(() => {
            index_1.affect(noop);
            index_1.affect(noop);
            const one = index_1.state(() => 1);
            const two = index_1.state(() => 2);
            const four = index_1.state(() => {
                index_1.affect(noop);
                return index_1.state(() => 4);
            });
            const eight = index_1.state(() => index_1.state(() => index_1.state(() => 8)));
            const sixteen = 16;
            return one + two + four + eight + sixteen;
        })();
        expect(val).toEqual(31);
    });
});
describe('except', () => {
    it('should rethrow interrupts', () => {
        expect(() => {
            try {
                index_1.state(() => null);
            }
            catch (e) {
                index_1.except(e);
            }
        }).toThrow();
    });
    it('should not affect exceptions', () => {
        try {
            throw new Error();
        }
        catch (e) {
            index_1.except(e);
        }
    });
});
